@@ -52,6 +52,9 @@
 #include <choqokbehaviorsettings.h>
 #include <kstandarddirs.h>
 #include <KSplashScreen>
+#include <KMenu>
+#include <uploadmediadialog.h>
+#include <knotifyconfigwidget.h>
 
 MainWindow::MainWindow()
     : Choqok::UI::MainWindow(), quickWidget(0), s_settingsDialog(0), m_splash(0), microblogCounter(0)
@@ -60,11 +63,10 @@ MainWindow::MainWindow()
     setAttribute ( Qt::WA_DeleteOnClose, false );
     setAttribute ( Qt::WA_QuitOnClose, false );
 
-    connect(this, SIGNAL(updateTimelines()), SLOT(slotUpdateTimelines()));
     if( Choqok::BehaviorSettings::showSplashScreen() ){
         KStandardDirs *stdDirs = KGlobal::dirs();
         QString img = stdDirs->findResource( "data", "choqok/images/splash_screen.png" );
-        kDebug()<<img;
+//         kDebug()<<img;
         QPixmap splashpix( img );
         if(splashpix.isNull())
             kDebug()<<"Pixmap is NULL";
@@ -81,7 +83,7 @@ MainWindow::MainWindow()
     setCentralWidget( mainWidget );
 
     sysIcon = new SysTrayIcon(this);
-    sysIcon->show();
+//     sysIcon->show();
     setupActions();
     statusBar()->show();
     setupGUI();
@@ -106,6 +108,8 @@ MainWindow::MainWindow()
     QTimer::singleShot(0, Choqok::PluginManager::self(), SLOT( loadAllPlugins() ) );
 //     Choqok::AccountManager::self()->loadAllAccounts();
     QTimer::singleShot(0, Choqok::AccountManager::self(), SLOT( loadAllAccounts() ) );
+    
+    connect(this, SIGNAL(updateTimelines()), SLOT(slotUpdateTimelines()));
 
     QPoint pos = Choqok::BehaviorSettings::position();
     if(pos.x() != -1 && pos.y() != -1) {
@@ -145,12 +149,12 @@ void MainWindow::newPluginAvailable( Choqok::Plugin *plugin )
     guiFactory()->addClient(plugin);
 }
 
-void MainWindow::nextTab(const QWheelEvent & event)
+void MainWindow::nextTab(int delta,Qt::Orientation orientation)
 {
   if(!isVisible())
     return;
   KTabWidget * widget = 0;
-  switch(event.orientation()) {
+  switch(orientation) {
   case Qt::Vertical:
     widget = mainWidget;
   break;
@@ -168,7 +172,7 @@ void MainWindow::nextTab(const QWheelEvent & event)
   int count = widget->count();
   int index = widget->currentIndex();
   int page;
-  if(event.delta() > 0) {
+  if(delta > 0) {
     page = index>0?index-1:count-1;
   } else {
     page = index<count-1?index+1:0;
@@ -178,7 +182,7 @@ void MainWindow::nextTab(const QWheelEvent & event)
 
 void MainWindow::setupActions()
 {
-    KStandardAction::quit( this, SLOT( slotQuit() ), actionCollection() );
+    KAction *actQuit = KStandardAction::quit( this, SLOT( slotQuit() ), actionCollection() );
     KAction *prefs = KStandardAction::preferences( this, SLOT( slotConfigChoqok() ), actionCollection() );
 //     KStandardAction::configureNotifications ( this, SLOT ( slotConfNotifications() ), actionCollection() );
 
@@ -203,16 +207,19 @@ void MainWindow::setupActions()
     markRead->setShortcut( KShortcut( Qt::CTRL | Qt::Key_R ) );
     connect( markRead, SIGNAL( triggered( bool ) ), this, SIGNAL( markAllAsRead()) );
 
-    KAction *showMain = new KAction( this );
+    showMain = new KAction( this );
     actionCollection()->addAction( QLatin1String( "toggle_mainwin" ), showMain );
     KShortcut toggleMainGlobalShortcut( Qt::CTRL | Qt::META | Qt::Key_C );
     showMain->setGlobalShortcut( toggleMainGlobalShortcut/*, KAction::DefaultShortcut, KAction::NoAutoloading*/ );
-    showMain->setText( i18n( "Minimize" ) );
+    if(this->isVisible())
+        showMain->setText( i18n( "Minimize" ) );
+    else
+        showMain->setText( i18n("Restore") );
     connect( showMain, SIGNAL( triggered( bool ) ), this, SLOT( toggleMainWindow() ) );
 
-//     KAction *manageAccounts = new KAction(KIcon("user-group-properties"), i18n("Manage Accounts..."), this);
-//     actionCollection()->addAction(QLatin1String("manage_accounts"), manageAccounts);
-//     connect( manageAccounts, SIGNAL(triggered(bool)), this, SLOT(slotManageAccounts()) );
+    KAction *act = KStandardAction::configureNotifications ( this, SLOT ( slotConfNotifications() ),
+                                                             actionCollection() );
+    actionCollection()->addAction ( "settings_notifications", act );
 
     KAction *enableUpdates = new KAction( i18n( "Enable Update Timer" ), this );
     enableUpdates->setCheckable( true );
@@ -234,25 +241,32 @@ void MainWindow::setupActions()
     connect( clearAvatarCache, SIGNAL( triggered() ),
              Choqok::MediaManager::self(), SLOT(clearImageCache()) );
 
+    KAction *uploadMedium = new KAction( KIcon("arrow-up"), i18n( "Upload Medium" ), this );
+    actionCollection()->addAction( QLatin1String( "choqok_upload_medium" ), uploadMedium );
+    connect( uploadMedium, SIGNAL( triggered(bool)), this, SLOT(slotUploadMedium()) );
+
     ///SysTray Actions:
     sysIcon->contextMenu()->addAction( newTwit );
-
+//     sysIcon->contextMenu()->addAction( uploadMedium );
     sysIcon->contextMenu()->addAction( actUpdate );
     sysIcon->contextMenu()->addSeparator();
-    sysIcon->contextMenu()->addSeparator();
     connect( enableUpdates, SIGNAL( toggled( bool ) ), sysIcon, SLOT( setTimeLineUpdatesEnabled( bool ) ) );
     sysIcon->contextMenu()->addAction( enableUpdates );
     sysIcon->setTimeLineUpdatesEnabled( enableUpdates->isChecked() );
 //     sysIcon->contextMenu()->addAction( enableNotify );
     sysIcon->contextMenu()->addAction( prefs );
 
-    connect( sysIcon, SIGNAL(quitSelected()), this, SLOT(slotQuit()) );
-    connect(sysIcon,SIGNAL(wheelEvent(const QWheelEvent&)),this,SLOT(nextTab(const QWheelEvent&)));
+    sysIcon->contextMenu()->addSeparator();
+    sysIcon->contextMenu()->addAction(showMain);
+    sysIcon->contextMenu()->addAction(actQuit);
+//     connect( sysIcon, SIGNAL(quitSelected()), this, SLOT(slotQuit()) );
+    connect(sysIcon, SIGNAL(scrollRequested(int,Qt::Orientation)),
+            this, SLOT(nextTab(int,Qt::Orientation)));
 }
 
 void MainWindow::slotConfNotifications()
 {
-//     KNotifyConfigWidget::configure ( this );
+    KNotifyConfigWidget::configure ( this );
 }
 
 void MainWindow::createQuickPostDialog()
@@ -262,6 +276,7 @@ void MainWindow::createQuickPostDialog()
     quickWidget->setAttribute(Qt::WA_DeleteOnClose, false);
     connect( quickWidget, SIGNAL( newPostSubmitted(Choqok::JobResult)),
              sysIcon, SLOT( slotJobDone(Choqok::JobResult)) );
+    emit(quickPostCreated());
 }
 
 void MainWindow::triggerQuickPost()
@@ -314,11 +329,14 @@ void MainWindow::slotAppearanceConfigChanged()
         Choqok::UI::PostWidget::setStyle( Choqok::AppearanceSettings::unreadForeColor() ,
                                           Choqok::AppearanceSettings::unreadBackColor(),
                                           Choqok::AppearanceSettings::readForeColor() ,
-                                          Choqok::AppearanceSettings::readBackColor());
+                                          Choqok::AppearanceSettings::readBackColor() ,
+                                         Choqok::AppearanceSettings::ownForeColor() ,
+                                         Choqok::AppearanceSettings::ownBackColor());
     } else {
         QPalette p = window()->palette();
         Choqok::UI::PostWidget::setStyle( p.color(QPalette::WindowText) , p.color(QPalette::Window).lighter() ,
-                                          p.color(QPalette::WindowText) , p.color(QPalette::Window));
+                                          p.color(QPalette::WindowText) , p.color(QPalette::Window) ,
+                                         p.color(QPalette::WindowText) , p.color(QPalette::Window));
     }
     int count = mainWidget->count();
     for ( int i = 0; i < count; ++i ) {
@@ -511,6 +529,18 @@ void MainWindow::toggleMainWindow()
         show();
 }
 
+void MainWindow::hideEvent(QHideEvent* event)
+{
+    Choqok::UI::MainWindow::hideEvent(event);
+    showMain->setText( i18n("Restore") );
+}
+
+void MainWindow::showEvent(QShowEvent* event)
+{
+    Choqok::UI::MainWindow::showEvent(event);
+    showMain->setText( i18n("Minimize") );
+}
+
 void MainWindow::slotMarkAllAsRead()
 {
     setWindowTitle( i18n("Choqok") );
@@ -544,7 +574,14 @@ void MainWindow::oneMicroblogLoaded()
 
 void MainWindow::slotUpdateTimelines()
 {
-    showStatusMessage(i18n("Loading timelines..."));
+    if ( Choqok::AccountManager::self()->accounts().count() )
+         showStatusMessage( i18n( "Loading timelines..." ) );
+}
+
+void MainWindow::slotUploadMedium()
+{
+    QPointer<Choqok::UI::UploadMediaDialog> dlg = new Choqok::UI::UploadMediaDialog(this);
+    dlg->show();
 }
 
 #include "mainwindow.moc"
