@@ -33,26 +33,31 @@
 #include <mediamanager.h>
 #include <choqokappearancesettings.h>
 #include "twitterapishowthread.h"
+#include <textbrowser.h>
+#include <KMessageBox>
+#include <KAction>
+#include <QMenu>
 
-const QRegExp TwitterApiPostWidget::mUserRegExp("([\\s]|^)@([^\\s\\W]+)");
-const QRegExp TwitterApiPostWidget::mHashtagRegExp("([\\s]|^)#([^\\s\\W]+)");
 const KIcon TwitterApiPostWidget::unFavIcon(Choqok::MediaManager::convertToGrayScale(KIcon("rating").pixmap(16)) );
 
 class TwitterApiPostWidget::Private
 {
 public:
-    Private()
+    Private(Choqok::Account* account)
         :isBasePostShowed(false)
-    {}
+    {
+        mBlog = qobject_cast<TwitterApiMicroBlog*>( account->microblog() );
+    }
     KPushButton *btnFav;
     bool isBasePostShowed;
+    TwitterApiMicroBlog *mBlog;
 };
 
 TwitterApiPostWidget::TwitterApiPostWidget(Choqok::Account* account, const Choqok::Post &post, QWidget* parent)
-    : PostWidget(account, post, parent), d(new Private)
+    : PostWidget(account, post, parent), d(new Private(account))
 {
-    document()->addResource( QTextDocument::ImageResource, QUrl("icon://thread"),
-                             KIcon("go-top").pixmap(8) );
+    mainWidget()->document()->addResource( QTextDocument::ImageResource, QUrl("icon://thread"),
+                             KIcon("go-top").pixmap(10) );
 }
 
 TwitterApiPostWidget::~TwitterApiPostWidget()
@@ -65,7 +70,25 @@ void TwitterApiPostWidget::initUi()
     Choqok::UI::PostWidget::initUi();
 
     KPushButton *btnRe = addButton( "btnReply",i18nc( "@info:tooltip", "Reply" ), "edit-undo" );
-    connect( btnRe, SIGNAL(clicked(bool)), SLOT(slotReply()) );
+    QMenu *menu = new QMenu(btnRe);
+
+    KAction *actRep = new KAction(KIcon("edit-undo"), i18n("Reply to %1", currentPost().author.userName), menu);
+    menu->addAction(actRep);
+    connect( actRep, SIGNAL(triggered(bool)), SLOT(slotReply()) );
+
+    KAction *actWrite = new KAction( KIcon("document-edit"), i18n("Write to %1", currentPost().author.userName),
+                                     menu );
+    menu->addAction(actWrite);
+    connect( actWrite, SIGNAL(triggered(bool)), SLOT(slotWriteTo()) );
+
+    if( !currentPost().isPrivate ) {
+        KAction *actReplytoAll = new KAction(i18n("Reply to all"), menu);
+        menu->addAction(actReplytoAll);
+        connect( actReplytoAll, SIGNAL(triggered(bool)), SLOT(slotReplyToAll()) );
+    }
+
+    menu->setDefaultAction(actRep);
+    btnRe->setMenu(menu);
 
     if( !currentPost().isPrivate ) {
         d->btnFav = addButton( "btnFavorite",i18nc( "@info:tooltip", "Favorite" ), "rating" );
@@ -77,12 +100,7 @@ void TwitterApiPostWidget::initUi()
 
 QString TwitterApiPostWidget::prepareStatus(const QString& text)
 {
-    QString res = Choqok::UI::PostWidget::prepareStatus(text);
-    res.replace(mUserRegExp,"\\1@<a href='user://\\2'>\\2</a> <a href='"+
-    currentAccount()->microblog()->profileUrl( currentAccount(), "\\2") + "' title='" +
-    currentAccount()->microblog()->profileUrl( currentAccount(), "\\2") + "'>"+ webIconText +"</a>");
-
-    return res;
+    return Choqok::UI::PostWidget::prepareStatus(text);
 }
 
 QString TwitterApiPostWidget::generateSign()
@@ -92,8 +110,7 @@ QString TwitterApiPostWidget::generateSign()
                                                                 currentPost().author.userName);
     sign = "<b><a href='user://"+currentPost().author.userName+"' title=\"" +
     currentPost().author.description + "\">" + currentPost().author.userName +
-    "</a> <a href=\"" + profUrl + "\" title=\"" + profUrl
-    + "\">"+ webIconText +"</a> - </b>";
+    "</a> - </b>";
     //<img src=\"icon://web\" />
     sign += "<a href=\"" + currentPost().link +
     "\" title=\"" + currentPost().creationDateTime.toString( Qt::DefaultLocaleLongDate ) + "\">%1</a>";
@@ -104,18 +121,36 @@ QString TwitterApiPostWidget::generateSign()
             sign.prepend( "To " );
         }
     } else {
-        if( !currentPost().source.isNull() )
-            sign += " - " + currentPost().source;
+        if( !currentPost().source.isNull() ) {
+            sign += " - ";
+            if(currentPost().source == "ostatus" && !currentPost().author.homePageUrl.isEmpty()) {
+                KUrl srcUrl(currentPost().author.homePageUrl);
+                sign += i18n( "<a href='%1' title='Sent from %2 via OStatus'>%2</a>",
+                              currentPost().author.homePageUrl,
+                              srcUrl.host());
+            } else {
+                sign += currentPost().source;
+            }
+        }
         if ( !currentPost().replyToPostId.isEmpty() ) {
             QString link = currentAccount()->microblog()->postUrl( currentAccount(), currentPost().replyToUserName,
                                                                    currentPost().replyToPostId );
             QString showConMsg = i18n("Show Conversation");
             QString threadlink = "thread://" + currentPost().postId;
-            sign += " - <a href='replyto://" + currentPost().replyToPostId + "'>" +
-            i18n("in reply to")+ "</a>&nbsp;<a href=\"" + link +  "\" title=\""+ link +"\">"+webIconText+"</a> ";
+            sign += " - " +
+            i18n("<a href='replyto://%1'>in reply to</a>&nbsp;<a href=\"%2\" title=\"%2\">%3</a>",
+                currentPost().replyToPostId, link, webIconText) + ' ';
             sign += "<a title=\""+ showConMsg +"\" href=\"" + threadlink + "\"><img src=\"icon://thread\" /></a>";
         }
     }
+
+    //ReTweet detection:
+    if( !currentPost().repeatedFromUsername.isEmpty() ){
+        QString retweet;
+        retweet += "<br/>"
+                +  d->mBlog->generateRepeatedByUserTooltip( QString("<a href='user://%1'>%2</a>").arg( currentPost().repeatedFromUsername).arg(currentPost().repeatedFromUsername) );
+        sign.append(retweet);
+    }
     sign.prepend("<p dir='ltr'>");
     sign.append( "</p>" );
     return sign;
@@ -123,18 +158,30 @@ QString TwitterApiPostWidget::generateSign()
 
 void TwitterApiPostWidget::slotReply()
 {
+    setReadInternal();
     if(currentPost().isPrivate){
         TwitterApiAccount *account= qobject_cast<TwitterApiAccount*>( currentAccount() );
-        TwitterApiMicroBlog *microblog = qobject_cast<TwitterApiMicroBlog*>( currentAccount()->microblog() );
-        microblog->showDirectMessageDialog( account, currentPost().author.userName );
+        d->mBlog->showDirectMessageDialog( account, currentPost().author.userName );
     } else {
         emit reply( QString("@%1").arg(currentPost().author.userName), currentPost().postId );
     }
 }
 
+void TwitterApiPostWidget::slotWriteTo()
+{
+    emit reply( QString("@%1").arg(currentPost().author.userName), QString() );
+}
+
+void TwitterApiPostWidget::slotReplyToAll()
+{
+    QString txt = QString("@%1").arg(currentPost().author.userName);
+    emit reply(txt, currentPost().postId);
+}
+
 void TwitterApiPostWidget::setFavorite()
 {
-    TwitterApiMicroBlog *mic = qobject_cast<TwitterApiMicroBlog*>(currentAccount()->microblog());
+    setReadInternal();
+    TwitterApiMicroBlog *mic = d->mBlog;
     if(currentPost().isFavorited){
         connect(mic, SIGNAL(favoriteRemoved(Choqok::Account*,QString)),
                 this, SLOT(slotSetFavorite(Choqok::Account*,QString)) );
@@ -154,10 +201,9 @@ void TwitterApiPostWidget::slotSetFavorite(Choqok::Account *theAccount, const QS
         tmp.isFavorited = !tmp.isFavorited;
         setCurrentPost(tmp);
         updateFavStat();
-        TwitterApiMicroBlog *mic = qobject_cast<TwitterApiMicroBlog*>(currentAccount()->microblog());
-        disconnect(mic, SIGNAL(favoriteRemoved(Choqok::Account*,QString)),
+        disconnect(d->mBlog, SIGNAL(favoriteRemoved(Choqok::Account*,QString)),
                    this, SLOT(slotSetFavorite(Choqok::Account*,QString)) );
-        disconnect(mic, SIGNAL(favoriteCreated(Choqok::Account*,QString)),
+        disconnect(d->mBlog, SIGNAL(favoriteCreated(Choqok::Account*,QString)),
                    this, SLOT(slotSetFavorite(Choqok::Account*,QString)) );
         ///TODO Notify!
     }
@@ -179,7 +225,7 @@ void TwitterApiPostWidget::checkAnchor(const QUrl & url)
     QString scheme = url.scheme();
     if( scheme == "replyto" ) {
         if(d->isBasePostShowed) {
-            setContent( prepareStatus(currentPost().content) );
+            setContent( prepareStatus(currentPost().content).replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive) );
             updateUi();
             d->isBasePostShowed = false;
             return;
@@ -220,15 +266,30 @@ void TwitterApiPostWidget::slotBasePostFetched(Choqok::Account* theAccount, Choq
             color = this->palette().dark().color().name();
         }
         QString baseStatusText = "<p style=\"margin-top:10px; margin-bottom:10px; margin-left:20px;\
-        margin-right:20px; -qt-block-indent:0; text-indent:0px\"><span style=\" color:" + color + ";\">";
+        margin-right:20px; text-indent:0px\"><span style=\" color:" + color + ";\">";
         baseStatusText += "<b><a href='user://"+ post->author.userName +"'>" +
         post->author.userName + "</a> :</b> ";
 
         baseStatusText += prepareStatus( post->content ) + "</p>";
-        setContent( content().prepend( baseStatusText ) );
+        setContent( content().prepend( baseStatusText.replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive) ) );
         updateUi();
-        delete post;
+//         delete post;
     }
 }
 
+void TwitterApiPostWidget::repeatPost()
+{
+    setReadInternal();
+    ChoqokId postId;
+    if(currentPost().repeatedPostId.isEmpty())
+        postId = currentPost().postId;
+    else
+        postId = currentPost().repeatedPostId;
+    if( KMessageBox::questionYesNo(Choqok::UI::Global::mainWindow(), d->mBlog->repeatQuestion(),
+                               QString(), KStandardGuiItem::yes(), KStandardGuiItem::cancel(),
+                               "dontAskRepeatConfirm") == KMessageBox::Yes )
+        d->mBlog->repeatPost(currentAccount(), postId);
+}
+
+
 #include "twitterapipostwidget.moc"
