@@ -21,6 +21,8 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, see http://www.gnu.org/licenses/
 */
 #include "postwidget.h"
+#include "choqoktools.h"
+#include "textbrowser.h"
 #include <qboxlayout.h>
 #include <KLocale>
 #include <KPushButton>
@@ -34,14 +36,10 @@ along with this program; if not, see http://www.gnu.org/licenses/
 #include <KToolInvocation>
 #include <KMessageBox>
 #include "choqokappearancesettings.h"
-#include <kaction.h>
-#include <QApplication>
-#include <QClipboard>
-#include <QContextMenuEvent>
 #include <QMenu>
 #include <kmenu.h>
-#include <qabstracttextdocumentlayout.h>
-#include "choqoktools.h"
+#include <QCloseEvent>
+#include <qmutex.h>
 
 static const int _15SECS = 15000;
 static const int _MINUTE = 60000;
@@ -65,39 +63,61 @@ class PostWidget::Private
 //         bool mRead;
         QTimer mTimer;
 
-        static const QString ownText;
-        static const QString otherText;
         //BEGIN UI contents:
         QString mSign;
         QString mContent;
         QString mImage;
         //END UI contents;
+
+        QStringList detectedUrls;
 };
 
-const QString PostWidget::Private::ownText ("<table width=\"100%\"><tr><td><p>%2</p></td><td rowspan=\"2\"\
-width=\"48\">%1</td></tr><tr><td style=\"font-size:small;\" align=\"right\">%3</td></tr></table>");
 
-const QString PostWidget::Private::otherText ( "<table width=\"100%\"><tr><td rowspan=\"2\"\
-width=\"48\">%1</td><td><p>%2</p></td></tr><tr><td style=\"font-size:small;\" align=\"right\">%3</td></tr></table>");
+const QString PostWidget::ownText ("<table width=\"100%\" ><tr><td><p>%2</p></td><td width=\"5\"><!-- empty --></td><td width=\"48\" rowspan=\"2\" align=\"right\">%1</td></tr><tr><td style=\"font-size:small;\" dir=\"ltr\" align=\"right\" valign=\"bottom\">%3</td></tr></table>");
+
+const QString PostWidget::otherText ( "<table height=\"100%\" width=\"100%\"><tr><td rowspan=\"2\"\
+width=\"48\">%1</td><td width=\"5\"><!-- EMPTY HAHA --></td><td><p>%2</p></td></tr><tr><td><!-- EMPTY HAHA --></td><td style=\"font-size:small;\" dir=\"ltr\" align=\"right\" width=\"100%\" valign=\"bottom\">%3</td></tr></table>");
+
 const QString PostWidget::baseStyle ("KTextBrowser {border: 1px solid rgb(150,150,150);\
-border-radius:5px;} KTextBrowser {color:%1; background-color:%2}\
+border-radius:5px;}  KTextBrowser {color:%1; background-color:%2}\
 KPushButton{border:0px}");
-const QRegExp PostWidget::mUrlRegExp("((ftps?|https?)://[^\\s<>\"]+[^!,\\.\\s<>'\"\\)\\]])");
+
+const QString protocols = "((https?|ftps?)://)";
+const QString subdomains = "(([a-z0-9-_]\\.)?)";
+const QString auth = "((([a-z0-9-_]{1,})((:[\\S]{1,})?)@)?)";
+const QString domains = "(([a-z0-9-\\x0080-\\xFFFF]){1,63}\\.)+";
+const QString port = "(:(6553[0-5]|655[0-2][0-9]|65[0-4][\\d]{2}|6[0-4][\\d]{3}|[1-5][\\d]{4}|[1-9][\\d]{0,3}))";
+const QString zone ("((a[cdefgilmnoqrstuwxz])|(b[abdefghijlmnorstvwyz])|(c[acdfghiklmnoruvxyz])|(d[ejkmoz])|(e[ceghrstu])|\
+(f[ijkmor])|(g[abdefghilmnpqrstuwy])|(h[kmnrtu])|(i[delmnoqrst])|(j[emop])|(k[eghimnprwyz])|(l[abcikrstuvy])|\
+(m[acdefghklmnopqrstuvwxyz])|(n[acefgilopruz])|(om)|(p[aefghklnrstwy])|(qa)|(r[eosuw])|(s[abcdeghijklmnortuvyz])|\
+(t[cdfghjkmnoprtvwz])|(u[agksyz])|(v[aceginu])|(w[fs])|(ye)|(z[amrw])\
+|(asia|com|info|net|org|biz|name|pro|aero|cat|coop|edu|jobs|mobi|museum|tel|travel|gov|int|mil|local)|(中国)|(公司)|(网络)|(صر)|(امارات)|(рф))");
+const QString ip = "(25[0-5]|[2][0-4][0-9]|[0-1]?[\\d]{1,2})(\\.(25[0-5]|[2][0-4][0-9]|[0-1]?[\\d]{1,2})){3}";
+const QString params = "(((\\/)[\\w:/\\?#\\[\\]@!\\$&\\(\\)\\*%\\+,;=\\._~\\-\\|]{1,}|%[0-9a-f]{2})?)";
+
+const QRegExp PostWidget::mUrlRegExp("((((" + protocols + "?)" + auth +
+                          subdomains +
+                          '(' + domains +               
+                          zone + "(?!(\\w))))|(" + protocols + '(' + ip + ")+))" +
+                          '(' + port + "?)" + "((\\/)?)"  +
+                          params + ')', Qt::CaseInsensitive);
+
 QString PostWidget::readStyle;
 QString PostWidget::unreadStyle;
+QString PostWidget::ownStyle;
 const QString PostWidget::webIconText("&#9755;");
 
 PostWidget::PostWidget( Account* account, const Choqok::Post& post, QWidget* parent/* = 0*/ )
-    :KTextBrowser(parent), d(new Private(account, post))
+    :QWidget(parent), _mainWidget(new TextBrowser(this)), d(new Private(account, post))
 {
     setAttribute(Qt::WA_DeleteOnClose);
+    _mainWidget->setFrameShape(QFrame::NoFrame);
     if(currentAccount()->username().compare( currentPost().author.userName, Qt::CaseInsensitive ) == 0 )
         d->mCurrentPost.isRead = true;
-    setupUi();
     d->mTimer.start( _MINUTE );
     connect( &d->mTimer, SIGNAL( timeout() ), this, SLOT( updateUi()) );
-    setOpenLinks(false);
-    connect(this,SIGNAL(anchorClicked(QUrl)),this,SLOT(checkAnchor(QUrl)));
+    connect(_mainWidget, SIGNAL(clicked(QMouseEvent*)), SLOT(mousePressEvent(QMouseEvent*)));
+    connect(_mainWidget, SIGNAL(anchorClicked(QUrl)), this, SLOT(checkAnchor(QUrl)));
 //     setTextInteractionFlags( Qt::TextSelectableByKeyboard | Qt::TextSelectableByMouse );
 }
 
@@ -108,6 +128,8 @@ void PostWidget::checkAnchor(const QUrl & url)
 
 PostWidget::~PostWidget()
 {
+  //Attribute "Qt::WA_DeleteOnClose" is enabled at construtor! So please use close() for deleting an object, instead of deleteLater() or delete
+    delete d;
 }
 
 Account* PostWidget::currentAccount()
@@ -135,55 +157,66 @@ QString PostWidget::generateSign()
 
 void PostWidget::setupUi()
 {
+    setLayout(new QVBoxLayout);
+    layout()->setMargin(0);
+    layout()->setContentsMargins(0,0,0,0);
+    layout()->addWidget(_mainWidget);
     setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Fixed);
-    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    _mainWidget->setFocusProxy(this);
 
-    d->buttonsLayout = new QGridLayout(this);
+    d->buttonsLayout = new QGridLayout(_mainWidget);
     d->buttonsLayout->setRowStretch(0,100);
     d->buttonsLayout->setColumnStretch(5,100);
     d->buttonsLayout->setMargin(0);
     d->buttonsLayout->setSpacing(0);
 
-    this->setLayout(d->buttonsLayout);
-    connect(this,SIGNAL(textChanged()),this,SLOT(setHeight()));
+    _mainWidget->setLayout(d->buttonsLayout);
+    connect(_mainWidget,SIGNAL(textChanged()),this,SLOT(setHeight()));
 
-    document()->addResource( QTextDocument::ImageResource, QUrl("img://profileImage"),
-                             MediaManager::self()->defaultImage() );
-                             d->mImage = "<img src=\"img://profileImage\" title=\""+
-                             d->mCurrentPost.author.realName + "\" width=\"48\" height=\"48\" />";
-
-    if(d->mCurrentAccount->username().compare( d->mCurrentPost.author.userName, Qt::CaseInsensitive ) == 0) {
-        KPushButton *btnRemove = addButton("btnRemove", i18nc( "@info:tooltip", "Remove" ), "edit-delete" );
-        connect(btnRemove, SIGNAL(clicked(bool)), SLOT(removeCurrentPost()));
-        baseText = &Private::ownText;
-    } else {
-        KPushButton *btnResend = addButton("btnResend", i18nc( "@info:tooltip", "ReSend" ), "retweet" );
-        connect(btnResend, SIGNAL(clicked(bool)), SLOT(slotResendPost()));
-        baseText = &Private::otherText;
-    }
 }
 
 void PostWidget::initUi()
 {
+    setupUi();
+    _mainWidget->document()->addResource( QTextDocument::ImageResource, QUrl("img://profileImage"),
+                             MediaManager::self()->defaultImage() );
+
+    if(d->mCurrentAccount->username().compare( d->mCurrentPost.author.userName, Qt::CaseInsensitive ) == 0
+        || currentPost().isPrivate) {
+        KPushButton *btnRemove = addButton("btnRemove", i18nc( "@info:tooltip", "Remove" ), "edit-delete" );
+        connect(btnRemove, SIGNAL(clicked(bool)), SLOT(removeCurrentPost()));
+        baseText = &ownText;
+    } else {
+        KPushButton *btnResend = addButton("btnResend", i18nc( "@info:tooltip", "ReSend" ), "retweet" );
+        connect(btnResend, SIGNAL(clicked(bool)), SLOT(slotResendPost()));
+        baseText = &otherText;
+    }
     d->mImage = "<img src=\"img://profileImage\" title=\""+ d->mCurrentPost.author.realName +"\" width=\"48\" height=\"48\" />";
     d->mContent = prepareStatus(d->mCurrentPost.content);
     d->mSign = generateSign();
     setupAvatar();
-    setDirection();
+    if(QLatin1String(qVersion()) < "4.7"){//Due Qt 4.7 detects and sets the direction//TODO remove this part at all
+        setDirection();
+    }
     setUiStyle();
+
+    d->mContent.replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive);
+    d->mSign.replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive);
+
     updateUi();
 }
 
 void PostWidget::updateUi()
 {
-    setHtml( baseText->arg( d->mImage, d->mContent, d->mSign.arg( formatDateTime( d->mCurrentPost.creationDateTime ) ) ) );
+    _mainWidget->setHtml(baseText->arg(d->mImage, d->mContent,
+                                      d->mSign.arg(formatDateTime( d->mCurrentPost.creationDateTime ))));
 }
 
-void PostWidget::setStyle(const QColor& color, const QColor& back, const QColor& read, const QColor& readBack)
+void PostWidget::setStyle(const QColor& color, const QColor& back, const QColor& read, const QColor& readBack, const QColor& own, const QColor& ownBack)
 {
     unreadStyle = baseStyle.arg( getColorString(color), getColorString(back) );
     readStyle = baseStyle.arg( getColorString(read), getColorString(readBack) );
+    ownStyle = baseStyle.arg( getColorString(own), getColorString(ownBack) );
 }
 
 QString PostWidget::getColorString(const QColor& color)
@@ -194,24 +227,13 @@ QString PostWidget::getColorString(const QColor& color)
 
 KPushButton * PostWidget::addButton(const QString & objName, const QString & toolTip, const QString & icon)
 {
-    KPushButton * button = new KPushButton(KIcon(icon), QString(), this);
-    button->setObjectName(objName);
-    button->setToolTip(toolTip);
-    button->setIconSize(QSize(16,16));
-    button->setMinimumSize(QSize(20, 20));
-    button->setMaximumSize(QSize(20, 20));
-    button->setFlat(true);
-    button->setVisible(false);
-    button->setCursor(Qt::PointingHandCursor);
+    return addButton(objName, toolTip, KIcon(icon));
 
-    d->mUiButtons.insert( objName, button );
-    d->buttonsLayout->addWidget( button, 1, d->mUiButtons.count() );
-    return button;
 }
 
 KPushButton * PostWidget::addButton(const QString & objName, const QString & toolTip, const KIcon & icon)
 {
-    KPushButton * button = new KPushButton(icon, QString(), this);
+    KPushButton * button = new KPushButton(icon, QString(), _mainWidget);
     button->setObjectName(objName);
     button->setToolTip(toolTip);
     button->setIconSize(QSize(16,16));
@@ -248,6 +270,14 @@ void PostWidget::setRead(bool read/* = true*/)
     }
 }
 
+void PostWidget::setReadInternal()
+{
+    if(!isRead()){
+        setRead();
+        Q_EMIT postReaded();
+    }
+}
+
 bool PostWidget::isRead() const
 {
     return currentPost().isRead;
@@ -255,68 +285,91 @@ bool PostWidget::isRead() const
 
 void PostWidget::setUiStyle()
 {
-    if(currentPost().isRead)
+    if (currentAccount()->username().compare( currentPost().author.userName, Qt::CaseInsensitive ) == 0)
+      setStyleSheet(ownStyle);
+    else {
+      if(currentPost().isRead)
         setStyleSheet(readStyle);
-    else
+      else
         setStyleSheet(unreadStyle);
+    }
 }
 
 void PostWidget::setHeight()
 {
-    document()->setTextWidth(width()-2);
-    int h = document()->size().toSize().height()+2;
+    _mainWidget->document()->setTextWidth(width()-2);
+    int h = _mainWidget->document()->size().toSize().height()+2;
     setMinimumHeight(h);
     setMaximumHeight(h);
 }
 
 void PostWidget::closeEvent(QCloseEvent* event)
 {
-    kDebug();
-    emit aboutClosing(currentPost().postId, this);
+    QMutex mt;
+    mt.lock();
+    if( !isRead() )
+        setReadInternal();
+    Q_EMIT aboutClosing(currentPost().postId, this);
     event->accept();
+    mt.unlock();
 }
 
 void PostWidget::mousePressEvent(QMouseEvent* ev)
 {
     if(!isRead()) {
-        kDebug()<<"Emitting postReaded()";
-        emit postReaded();
-        setRead();
+        setReadInternal();
     }
-    KTextBrowser::mousePressEvent(ev);
+    QWidget::mousePressEvent(ev);
 }
 
 void PostWidget::resizeEvent ( QResizeEvent * event )
 {
     setHeight();
-    KTextBrowser::resizeEvent(event);
+    QWidget::resizeEvent(event);
 }
 
 void PostWidget::enterEvent ( QEvent * event )
 {
     foreach(KPushButton *btn, buttons()){
-        btn->show();
+        if(btn)//A crash happens here :/
+            btn->show();
     }
-    KTextBrowser::enterEvent(event);
+    QWidget::enterEvent(event);
 }
 
 void PostWidget::leaveEvent ( QEvent * event )
 {
     foreach(KPushButton *btn, buttons()){
-        btn->hide();
+        if(btn)
+            btn->hide();
     }
-    KTextBrowser::enterEvent(event);
+    QWidget::enterEvent(event);
 }
 
 QString PostWidget::prepareStatus( const QString &txt )
 {
     QString text = txt;
+    text.replace( "&amp;", "&amp;amp;" );
     text.replace( '<', "&lt;" );
     text.replace( '>', "&gt;" );
-    text.replace( " www.", " http://www." );
-    if ( text.startsWith( QLatin1String("www.") ) )
-        text.prepend( "http://" );
-    text.replace(mUrlRegExp,"<a href='\\1' title='\\1'>\\1</a>");
+    int pos = 0;
+    while(((pos = mUrlRegExp.indexIn(text, pos)) != -1)) {
+        QString link = mUrlRegExp.cap(0);
+        QString tmplink = link;
+        if ( (pos - 1 > -1 && ( text.at( pos - 1 ) != '@' && 
+             text.at( pos - 1 ) != '#' && text.at( pos - 1 ) != '!')) ||
+             pos == 0 ) { 
+        text.remove( pos, link.length() );
+        d->detectedUrls << link;
+        if ( !tmplink.startsWith(QLatin1String("http"), Qt::CaseInsensitive) &&
+             !tmplink.startsWith(QLatin1String("ftp"), Qt::CaseInsensitive) )
+             tmplink.prepend("http://");
+        static const QString hrefTemplate("<a href='%1' title='%1' target='_blank'>%2</a>");
+        tmplink = hrefTemplate.arg( tmplink, link );
+        text.insert( pos, tmplink );
+	}
+        pos += tmplink.length();
+    }
 
     if(AppearanceSettings::isEmoticonsEnabled())
         text = MediaManager::self()->parseEmoticons(text);
@@ -331,14 +384,20 @@ void PostWidget::setDirection()
     txt.remove(QRegExp("@([^\\s\\W]+)"));
     txt.remove(QRegExp("#([^\\s\\W]+)"));
     txt.remove(QRegExp("!([^\\s\\W]+)"));
-    txt.prepend(' ');
+    txt = txt.trimmed();
+//     kDebug()<<txt<<txt.isRightToLeft();
     if( txt.isRightToLeft() ) {
-        QTextOption options(document()->defaultTextOption());
+        QTextOption options(_mainWidget->document()->defaultTextOption());
         options.setTextDirection( Qt::RightToLeft );
-        document()->setDefaultTextOption(options);
+        _mainWidget->document()->setDefaultTextOption(options);
     }
 }
 
+QString PostWidget::formatDateTime(const KDateTime& time)
+{
+    return formatDateTime(time.dateTime());
+}
+
 QString PostWidget::formatDateTime( const QDateTime& time )
 {
     int seconds = time.secsTo( QDateTime::currentDateTime() );
@@ -377,6 +436,7 @@ void PostWidget::removeCurrentPost()
         connect( d->mCurrentAccount->microblog(),
                 SIGNAL(errorPost(Choqok::Account*, Choqok::Post*,Choqok::MicroBlog::ErrorType,QString)),
                 this, SLOT(slotPostError(Choqok::Account*, Choqok::Post*,Choqok::MicroBlog::ErrorType,QString)) );
+        setReadInternal();
         d->mCurrentAccount->microblog()->removePost(d->mCurrentAccount, &d->mCurrentPost);
     }
 }
@@ -390,24 +450,24 @@ void PostWidget::slotCurrentPostRemoved( Account* theAccount, Post* post )
 void PostWidget::slotResendPost()
 {
     QString text = generateResendText();
-
+    setReadInternal();
     if((BehaviorSettings::resendWithQuickPost() || currentAccount()->isReadOnly()) && Global::quickPostWidget())
         Global::quickPostWidget()->setText(text);
     else
-        emit resendPost(text);
+        Q_EMIT resendPost(text);
 }
 
 QString PostWidget::generateResendText()
 {
-	if (BehaviorSettings::useCustomRT())
-	{
-		return QString(BehaviorSettings::customRT()) + " @" + currentPost().author.userName + ": " + currentPost().content;
-	}
-	else
-	{
-		QChar re(0x267B);
-		return QString(re) + " @" + currentPost().author.userName + ": " + currentPost().content;
-	}
+    if (BehaviorSettings::useCustomRT())
+    {
+        return QString(BehaviorSettings::customRT()) + " @" + currentPost().author.userName + ": " + currentPost().content;
+    }
+    else
+    {
+        QChar re(0x267B);
+        return QString(re) + " @" + currentPost().author.userName + ": " + currentPost().content;
+    }
 }
 
 void PostWidget::setupAvatar()
@@ -428,7 +488,7 @@ void PostWidget::avatarFetched(const QString& remoteUrl, const QPixmap& pixmap)
 {
     if ( remoteUrl == d->mCurrentPost.author.profileImageUrl ) {
         QString url = "img://profileImage";
-        document()->addResource( QTextDocument::ImageResource, url, pixmap );
+        _mainWidget->document()->addResource( QTextDocument::ImageResource, url, pixmap );
         updateUi();
         disconnect( MediaManager::self(), SIGNAL( imageFetched(QString,QPixmap)),
                     this, SLOT(avatarFetched(QString, QPixmap) ) );
@@ -443,7 +503,8 @@ void PostWidget::avatarFetchError(const QString& remoteUrl, const QString& errMs
     if( remoteUrl == d->mCurrentPost.author.profileImageUrl ){
         ///Avatar fetching is failed! but will not disconnect to get the img if it fetches later!
         QString url = "img://profileImage";
-        document()->addResource( QTextDocument::ImageResource, url, KIcon("image-missing").pixmap(48) );
+        _mainWidget->document()->addResource( QTextDocument::ImageResource,
+                                             url, KIcon("image-missing").pixmap(48) );
         updateUi();
     }
 }
@@ -466,46 +527,6 @@ void PostWidget::slotPostError(Account* theAccount, Choqok::Post* post,
     }
 }
 
-void PostWidget::contextMenuEvent(QContextMenuEvent* event)
-{
-    KMenu *menu = new KMenu(this);
-    KAction *copy = new KAction( i18n("Copy"), this );
-//     copy->setShortcut( KShortcut( Qt::ControlModifier | Qt::Key_C ) );
-    connect( copy, SIGNAL(triggered(bool)), SLOT(slotCopyPostContent()) );
-    menu->addAction(copy);
-    QString anchor = document()->documentLayout()->anchorAt(event->pos());
-    if( !anchor.isEmpty() ){
-        KAction *copyLink = new KAction( i18n("Copy Link Location"), this );
-        copyLink->setData( anchor );
-        connect( copyLink, SIGNAL(triggered(bool)), SLOT(slotCopyLink()) );
-        menu->addAction(copyLink);
-    }
-    menu->addSeparator();
-    KAction *selectAll = new KAction(i18n("Select All"), this);
-//     selectAll->setShortcut( KShortcut( Qt::ControlModifier | Qt::Key_A ) );
-    connect( selectAll, SIGNAL(triggered(bool)), SLOT(selectAll()) );
-    menu->addAction(selectAll);
-    menu->popup(event->globalPos());
-}
-
-void PostWidget::slotCopyPostContent()
-{
-    QString txt = textCursor().selectedText();
-    if( txt.isEmpty() )
-        QApplication::clipboard()->setText( currentPost().content );
-    else
-        QApplication::clipboard()->setText( txt );
-}
-
-void PostWidget::slotCopyLink()
-{
-    KAction *act = qobject_cast< KAction* >( sender() );
-    if( act ){
-        QString link = act->data().toString();
-        QApplication::clipboard()->setText( link );
-    }
-}
-
 QString PostWidget::avatarText() const
 {
     return d->mImage;
@@ -528,6 +549,11 @@ void PostWidget::setContent(const QString& content)
     updateUi();
 }
 
+QStringList PostWidget::urls()
+{
+    return d->detectedUrls;
+}
+
 QString PostWidget::sign() const
 {
     return d->mSign;
@@ -544,4 +570,50 @@ void PostWidget::deleteLater()
     close();
 }
 
+TextBrowser* PostWidget::mainWidget()
+{
+    return _mainWidget;
+}
+
+void PostWidget::wheelEvent(QWheelEvent* event)
+{
+    event->ignore();
+}
+
+void PostWidget::addAction(KAction* action)
+{
+    TextBrowser::addAction(action);
+}
+
+class PostWidgetUserData::Private
+{
+public:
+    Private(PostWidget *postwidget)
+    :postWidget(postwidget)
+    {}
+    PostWidget *postWidget;
+};
+
+PostWidgetUserData::PostWidgetUserData( PostWidget *postWidget )
+:QObjectUserData(), d(new Private(postWidget))
+{
+
+}
+
+PostWidgetUserData::~PostWidgetUserData()
+{
+    delete d;
+}
+
+PostWidget* PostWidgetUserData::postWidget()
+{
+    return d->postWidget;
+}
+
+void PostWidgetUserData::setPostWidget(PostWidget* widget)
+{
+    d->postWidget = widget;
+}
+
+
 #include "postwidget.moc"
