@@ -23,25 +23,29 @@ along with this program; if not, see http://www.gnu.org/licenses/
 #ifndef POSTWIDGET_H
 #define POSTWIDGET_H
 
-#include <ktextbrowser.h>
+#include <QtCore/QTimer>
+#include <KDE/KDateTime>
+#include <kicon.h>
 #include <account.h>
 #include <choqoktypes.h>
 #include <microblog.h>
-#include <QTimer>
-#include <kicon.h>
 
+class KAction;
 class QGridLayout;
 class KPushButton;
 
 namespace Choqok {
 namespace UI {
+
+class TextBrowser;
+
 /**
 Post Widget!
 Attribute "Qt::WA_DeleteOnClose" is enabled at construtor! So please use close() for deleting an object, instead of deleteLater() or delete
 
 @author Mehrdad Momeny \<mehrdad.momeny@gmail.com\>
 */
-class CHOQOK_EXPORT PostWidget : public KTextBrowser
+class CHOQOK_EXPORT PostWidget : public QWidget
 {
     Q_OBJECT
     Q_PROPERTY (bool read READ isRead)
@@ -49,8 +53,8 @@ public:
     explicit PostWidget( Account *account, const Post &post, QWidget *parent = 0 );
     virtual ~PostWidget();
     const Post &currentPost() const;
-    void setRead(bool read = true);
-    bool isRead() const;
+    virtual void setRead(bool read = true);
+    virtual bool isRead() const;
 
     Account *currentAccount();
     /**
@@ -65,7 +69,8 @@ public:
     @see setUiStyle()
     */
     static void setStyle(const QColor& unreadColor, const QColor& unreadBack,
-                          const QColor& readColor, const QColor& readBack);
+                          const QColor& readColor, const QColor& readBack,
+                          const QColor& ownColor, const QColor& ownBack);
 
     /**
     @brief Set current post
@@ -101,14 +106,25 @@ public:
     QString content() const;
 
     void deleteLater();
-public slots:
+
+    TextBrowser * mainWidget();
+
+    QStringList urls();
+
+    /**
+     * Plugins can add status specific actions and process them internally
+     * 
+     */
+    static void addAction( KAction *action );
+
+public Q_SLOTS:
     /**
     Set Style sheet of widget to corresponding data->
     @see setStyle()
     */
     void setUiStyle();
 
-signals:
+Q_SIGNALS:
     /**
     Emit and contain text to resend.
     */
@@ -128,17 +144,13 @@ signals:
     */
     void aboutClosing( const ChoqokId &postId, PostWidget *widget);
 
-protected slots:
-    /**
-        @brief Copy post text to clipboard.
-    */
-    void slotCopyPostContent();
+protected Q_SLOTS:
 
     virtual void checkAnchor(const QUrl & url);
     /**
     Set height of widget related to text contents
     */
-    void setHeight();
+    virtual void setHeight();
 
     /**
     Update UI after changes, such as timestamp
@@ -164,12 +176,12 @@ protected slots:
     void avatarFetchError( const QString &remoteUrl, const QString &errMsg );
     void avatarFetched( const QString &remoteUrl, const QPixmap &pixmap );
 
-    void slotCopyLink();
-protected:
-    virtual void closeEvent(QCloseEvent* event);
-    virtual void contextMenuEvent(QContextMenuEvent* event);
-    virtual void setupAvatar();
     virtual void mousePressEvent(QMouseEvent* ev);
+protected:
+    virtual void setupUi();
+    virtual void closeEvent(QCloseEvent* event);
+    virtual void setupAvatar();
+    virtual void wheelEvent(QWheelEvent* );
     virtual void resizeEvent ( QResizeEvent * event );
     virtual void enterEvent ( QEvent * event );
     virtual void leaveEvent ( QEvent * event );
@@ -177,6 +189,7 @@ protected:
     virtual void setDirection();
     virtual QString generateSign();
     virtual QString formatDateTime( const QDateTime &time );
+    virtual QString formatDateTime( const KDateTime &time );
     static QString getColorString(const QColor& color);
     /**
     @brief Create and Add a new button to widget
@@ -187,23 +200,46 @@ protected:
     KPushButton * addButton(const QString & objName, const QString & toolTip, const KIcon & icon);
     QMap<QString, KPushButton*> &buttons();
 
+    /**
+    Sets post widget as read, and emits postReaded() signal
+    Use this from inside PostWidget class and its subclasses
+    */
+    void setReadInternal();
+
 protected:
+    TextBrowser *_mainWidget;
     const QString *baseText;
     static const QString baseStyle;
     static QString readStyle;
     static QString unreadStyle;
+    static QString ownStyle;
     static const QRegExp mUrlRegExp;
     static const QString webIconText;
+    static const QString ownText;
+    static const QString otherText;
 
     void setAvatarText( const QString &text );
     QString avatarText() const;
     virtual QString generateResendText();
 
 private:
-    void setupUi();
     class Private;
-    Private *d;
+    Private *const d;
 };
+
+class CHOQOK_EXPORT PostWidgetUserData : public QObjectUserData
+{
+public:
+    PostWidgetUserData( PostWidget *postWidget );
+    virtual ~PostWidgetUserData();
+    PostWidget *postWidget();
+    void setPostWidget(PostWidget *widget);
+
+private:
+    class Private;
+    Private * const d;
+};
+
 }
 }
 #endif // POSTWIDGET_H
