@@ -47,15 +47,19 @@ along with this program; if not, see http://www.gnu.org/licenses/
 #include <KMessageBox>
 #include <kmimetype.h>
 #include "laconicacomposerwidget.h"
+#include "mediamanager.h"
+#include <choqokappearancesettings.h>
 
 K_PLUGIN_FACTORY( MyPluginFactory, registerPlugin < LaconicaMicroBlog > (); )
 K_EXPORT_PLUGIN( MyPluginFactory( "choqok_laconica" ) )
 
 LaconicaMicroBlog::LaconicaMicroBlog ( QObject* parent, const QVariantList&  )
-: TwitterApiMicroBlog(MyPluginFactory::componentData(), parent)
+: TwitterApiMicroBlog(MyPluginFactory::componentData(), parent), friendsPage(1)
 {
     kDebug();
     setServiceName("StatusNet");
+    mTimelineInfos["ReTweets"]->name = i18nc("Timeline name", "Repeated");
+    mTimelineInfos["ReTweets"]->description = i18nc("Timeline description", "Your posts that were repeated by others");
 //     setServiceHomepageUrl("http://twitter.com/");
 }
 
@@ -111,6 +115,16 @@ Choqok::UI::ComposerWidget* LaconicaMicroBlog::createComposerWidget(Choqok::Acco
 QString LaconicaMicroBlog::profileUrl( Choqok::Account *account, const QString &username) const
 {
     TwitterApiAccount *acc = qobject_cast<TwitterApiAccount*>(account);
+    if(username.contains('@')){
+        QStringList lst = username.split('@', QString::SkipEmptyParts);
+        if(lst.count() == 2){
+            if(lst[1].endsWith(QString(".status.net"))){
+                return QString("http://").arg(lst[1]);
+            } else {
+                return QString("http://%1/%2").arg(lst[1]).arg(lst[0]);
+            }
+        }
+    }
     if(acc){
         return QString( acc->homepageUrl().prettyUrl(KUrl::AddTrailingSlash) + username) ;
     } else
@@ -138,7 +152,7 @@ TwitterApiSearch* LaconicaMicroBlog::searchBackend()
 }
 
 void LaconicaMicroBlog::createPostWithAttachment(Choqok::Account* theAccount, Choqok::Post* post,
-                                   const QString& mediumToAttach)
+                                                 const QString& mediumToAttach)
 {
     if( mediumToAttach.isEmpty() ){
         TwitterApiMicroBlog::createPost(theAccount, post);
@@ -161,45 +175,25 @@ void LaconicaMicroBlog::createPostWithAttachment(Choqok::Account* theAccount, Ch
             return;
         }
         ///Documentation: http://identi.ca/notice/17779990
-        KUrl url = apiUrl( qobject_cast<TwitterApiAccount*>(theAccount) );
+        TwitterApiAccount* account = qobject_cast<TwitterApiAccount*>(theAccount);
+        KUrl url = account->apiUrl();
         url.addPath ( "/statuses/update.xml" );
-        QByteArray newLine("\r\n");
-        QString formHeader( newLine + "Content-Disposition: form-data; name=\"%1\"" );
-        QByteArray header(newLine + "--AaB03x");
-        QByteArray footer(newLine + "--AaB03x--");
         QByteArray fileContentType = KMimeType::findByUrl( picUrl, 0, true )->name().toUtf8();
-        QByteArray fileHeader(newLine + "Content-Disposition: file; name=\"media\"; filename=\"" +
-        picUrl.fileName().toUtf8()+"\"");
 
-        QByteArray data;
-//         if ( !post->replyToPostId.isEmpty() && post->content.indexOf ( '@' ) > -1 ) {
-//             data += "&in_reply_to_status_id=";
-//             data += post->replyToPostId.toLocal8Bit();
-//         }
-//         data += "&source=choqok";
+        QMap<QString, QByteArray> formdata;
+        formdata["status"] = post->content.toUtf8();
+        formdata["in_reply_to_status_id"] = post->replyToPostId.toLatin1();
+        formdata["source"] = "choqok";
 
-        data.append(header);
-        data.append(fileHeader);
-        data.append(newLine + "Content-Type: " + fileContentType);
-        data.append(newLine);
-        data.append(newLine + picData);
+        QMap<QString, QByteArray> mediafile;
+        mediafile["name"] = "media";
+        mediafile["filename"] = picUrl.fileName().toUtf8();
+        mediafile["mediumType"] = fileContentType;
+        mediafile["medium"] = picData;
+        QList< QMap<QString, QByteArray> > listMediafiles;
+        listMediafiles.append(mediafile);
 
-        data.append(header);
-        data.append(formHeader.arg("status").toLatin1());
-        data.append(newLine);
-        data.append(newLine + post->content.toUtf8() );
-
-        data.append(header);
-        data.append(formHeader.arg("in_reply_to_status_id").toLatin1());
-        data.append(newLine);
-        data.append(newLine + post->replyToPostId.toLatin1());
-
-        data.append(header);
-        data.append(formHeader.arg("source").toLatin1());
-        data.append(newLine);
-        data.append(newLine + "choqok");
-
-        data.append(footer);
+        QByteArray data = Choqok::MediaManager::createMultipartFormData(formdata, listMediafiles);
 
         KIO::StoredTransferJob *job = KIO::storedHttpPost(data, url, KIO::HideProgressInfo) ;
         if ( !job ) {
@@ -207,6 +201,7 @@ void LaconicaMicroBlog::createPostWithAttachment(Choqok::Account* theAccount, Ch
             return;
         }
         job->addMetaData( "content-type", "Content-Type: multipart/form-data; boundary=AaB03x" );
+        job->addMetaData("customHTTPHeader", "Authorization: " + authorizationHeader(account, url, QOAuth::POST));
         mCreatePostMap[ job ] = post;
         mJobsAccount[job] = theAccount;
         connect( job, SIGNAL( result( KJob* ) ),
@@ -215,4 +210,111 @@ void LaconicaMicroBlog::createPostWithAttachment(Choqok::Account* theAccount, Ch
     }
 }
 
+QString LaconicaMicroBlog::generateRepeatedByUserTooltip(const QString& username)
+{
+    if( Choqok::AppearanceSettings::showRetweetsInChoqokWay() )
+        return i18n("Repeat of %1", username);
+    else
+        return i18n("Repeated by %1", username);
+}
+
+QString LaconicaMicroBlog::repeatQuestion()
+{
+    return i18n("Repeat this notice?");
+}
+
+void LaconicaMicroBlog::listFriendsUsername(TwitterApiAccount* theAccount)
+{
+    friendsList.clear();
+    if ( theAccount ) {
+        requestFriendsScreenName(theAccount);
+    }
+}
+
+void LaconicaMicroBlog::requestFriendsScreenName(TwitterApiAccount* theAccount, int page)
+{
+    kDebug();
+    TwitterApiAccount* account = qobject_cast<TwitterApiAccount*>(theAccount);
+    KUrl url = account->apiUrl();
+    url.addPath( QString("/statuses/friends.%1").arg(format));
+    QOAuth::ParamMap params;
+    if(account->usingOAuth()){
+        params.insert( "page", QByteArray::number( page ) );
+    } else {
+        url.addQueryItem( "page", QString::number( page ) );
+    }
+
+    KIO::StoredTransferJob *job = KIO::storedGet( url, KIO::Reload, KIO::HideProgressInfo ) ;
+    if ( !job ) {
+        kDebug() << "Cannot create an http GET request!";
+        return;
+    }
+    job->addMetaData("customHTTPHeader", "Authorization: " + authorizationHeader(account, url, QOAuth::GET, params));
+    mJobsAccount[job] = theAccount;
+    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotRequestFriendsScreenName(KJob*) ) );
+    job->start();
+}
+
+void LaconicaMicroBlog::slotRequestFriendsScreenName(KJob* job)
+{
+    kDebug();
+    TwitterApiAccount *theAccount = qobject_cast<TwitterApiAccount *>( mJobsAccount.take(job) );
+    KIO::StoredTransferJob* stJob = qobject_cast<KIO::StoredTransferJob*>( job );
+    QStringList newList;
+    if(format=="json"){
+        newList = readUsersScreenNameFromJson( theAccount, stJob->data() );
+    } else {
+        newList = readUsersScreenNameFromXml( theAccount, stJob->data() );
+    }
+    friendsList << newList;
+    if ( newList.count() == 100 ) {
+        requestFriendsScreenName( theAccount, ++friendsPage );
+    } else {
+        friendsList.removeDuplicates();
+        theAccount->setFriendsList(friendsList);
+        emit friendsUsernameListed( theAccount, friendsList );
+    }
+}
+
+QStringList LaconicaMicroBlog::readUsersScreenNameFromXml(Choqok::Account* theAccount, const QByteArray& buffer)
+{
+    kDebug();
+    QStringList list;
+    QDomDocument document;
+    document.setContent( buffer );
+    QDomElement root = document.documentElement();
+
+    if ( root.tagName() != "users" ) {
+        QString err = checkXmlForError(buffer);
+        if(!err.isEmpty()){
+            emit error(theAccount, ServerError, err, Critical);
+        } else {
+            err = i18n( "Retrieving the friends list failed. The data returned from the server is corrupted." );
+            kDebug() << "there's no users tag in XML\t the XML is: \n" << buffer;
+            emit error(theAccount, ParsingError, err, Critical);
+            list<<QString(' ');
+        }
+        return list;
+    }
+    QDomNode node = root.firstChild();
+    QString timeStr;
+    while ( !node.isNull() ) {
+        if ( node.toElement().tagName() != "user" ) {
+            kDebug() << "there's no user tag in XML!\n"<<buffer;
+            return list;
+        }
+        QDomNode node2 = node.firstChild();
+        while ( !node2.isNull() ) {
+            if ( node2.toElement().tagName() == "screen_name" ) {
+                list.append( node2.toElement().text() );
+                break;
+            }
+            node2 = node2.nextSibling();
+        }
+        node = node.nextSibling();
+    }
+    return list;
+}
+
+
 #include "laconicamicroblog.moc"
